/*
 * Tiedoston luku promisen avulla kahdessa vaiheessa.
 * Käytetään ES6-syntaksia. Huomaa että tässä tapauksessa
 * callbackin koodi on yksinkertaisempi kuin promisen koodi
 * ja callback hoitaa homman ihan yhtä hyvin.
 */
const fs = require('fs'); //filesystem -modulin käyttöönotto

//Tehdään funktio joka luo ja palauttaa promisen
function readFilePromisified(filename) {
  //luodaan ja palautetaan promise
  return new Promise(function (fulfill, reject) {
    fs.readFile(filename, function (err, text) {
      if (err) {
        reject(err);
      } else {
        fulfill(text); // Tässä on promisen sisältö
      }
    });
  });
}

const jsonfile = __dirname + '\\config.json';

//käytetään promisea
readFilePromisified(jsonfile)
  .then(function (text) {
    //.then vastaanottaa promisen käsiteltäväksi
    const obj = JSON.parse(text);
    console.log(JSON.stringify(obj, null, 4));
  })
  .catch(function (error) {
    console.error('An error occurred', error);
  });

//Milloin sitten kannattaa käyttää promisea?

/*A good general rule in programming is to use the simplest reasonable tool for the job,
but no simpler. A callback is a simpler tool than promises, if the problem is equally
simple. If you're just doing a single request that is always asynchronous,
a callback will do fine. However, if you are doing a few requests at the same
time and need to mix their results together, then promises will make your final
solution simpler than the callback solution, because promises are good at that.*/

//Tällaisessa tilanteessa promise on yksinkertaisempi ratkaisu:

/*
        function fetchImage(url){
        return new Promise(function(resolve, reject){
        var img = new Image()
        img.onload = function(){
            resolve(url)
        }
        img.onerror = function(){
            reject(url)
        }
        img.src = url
    })
}

fetchImage('image1.png').then(function(url){
    console.log(url + ' downloaded!')
    return fetchImage('image2.png')
}).then(function(url){
    console.log(url + ' downloaded!')
    return fetchImage('image3.png')
}).then(function(url){
    console.log(url + ' downloaded!')
    return fetchImage('image4.png')
}).then(function(url){
    console.log(url + ' downloaded!')
})

//Console log:
// image1.png downloaded!
// image2.png downloaded!
// image3.png downloaded!
// image4.png downloaded!
*/
